@@ -825,6 +825,11 @@ BOOL NV_API_CALL os_pat_supported(void)
     return TRUE;
 }
 
+BOOL NV_API_CALL os_is_efi_enabled(void)
+{
+    return FALSE;
+}
+
 void NV_API_CALL os_register_compatible_ioctl(
     NvU32 cmd,
     NvU32 size
