@@ -83,7 +83,7 @@ int nvidia_attach(device_t dev)
             if (!sc->BAR_objects[i])
                 goto failed;
 
-            VM_OBJECT_LOCK(sc->BAR_objects[i]);
+            VM_OBJECT_WLOCK(sc->BAR_objects[i]);
             switch (i) {
                 case NV_GPU_BAR_INDEX_FB:
                     vm_object_set_memattr(sc->BAR_objects[i],
@@ -95,7 +95,7 @@ int nvidia_attach(device_t dev)
                             VM_MEMATTR_UNCACHEABLE);
                     break;
             }
-            VM_OBJECT_UNLOCK(sc->BAR_objects[i]);
+            VM_OBJECT_WUNLOCK(sc->BAR_objects[i]);
         }
     }
 
@@ -541,9 +541,9 @@ int nvidia_open_dev(
                 goto failed;
             }
 
-            VM_OBJECT_LOCK(sc->UD_object);
+            VM_OBJECT_WLOCK(sc->UD_object);
             vm_object_set_memattr(sc->UD_object, VM_MEMATTR_UNCACHEABLE);
-            VM_OBJECT_UNLOCK(sc->UD_object);
+            VM_OBJECT_WUNLOCK(sc->UD_object);
         }
 
         nv->flags |= NV_FLAG_OPEN;
@@ -1056,7 +1056,7 @@ found:
 
     if (at->alloc_type_contiguous) {
         *private = NULL;
-        return (void *)(at->pte_array[0].virtual_address +
+        return (void *)(NvUPtr)(at->pte_array[0].virtual_address +
                     (i * PAGE_SIZE) + offset);
     }
 
@@ -1066,7 +1066,7 @@ found:
     virtual_address = vm_map_min(kernel_map);
 
     status = vm_map_find(kernel_map, at->object, (i * PAGE_SIZE),
-            &virtual_address, size, VMFS_ANY_SPACE,
+            &virtual_address, size, 0, VMFS_ANY_SPACE,
             (VM_PROT_READ | VM_PROT_WRITE),
             (VM_PROT_READ | VM_PROT_WRITE), 0);
     if (status != KERN_SUCCESS) {
@@ -1181,7 +1181,7 @@ NvS32 nv_alloc_contig_pages(
         return ENOMEM;
     }
 
-    address = kmem_alloc_contig(kernel_map, size, flags, 0,
+    address = kmem_alloc_contig(kmem_arena, size, flags, 0,
             sc->dma_mask, PAGE_SIZE, 0, attr);
     if (!address) {
         status = ENOMEM;
@@ -1211,9 +1211,9 @@ NvS32 nv_alloc_contig_pages(
         goto failed;
     }
 
-    VM_OBJECT_LOCK(at->object);
+    VM_OBJECT_WLOCK(at->object);
     vm_object_set_memattr(at->object, attr);
-    VM_OBJECT_UNLOCK(at->object);
+    VM_OBJECT_WUNLOCK(at->object);
 
     *private = at;
     SLIST_INSERT_HEAD(&sc->alloc_list, at, list);
@@ -1230,8 +1230,7 @@ failed:
         os_flush_cpu_cache();
 
     if (at->pte_array[0].virtual_address != NULL) {
-        kmem_free(kernel_map,
-                at->pte_array[0].virtual_address, at->size);
+        kmem_free(kmem_arena, at->pte_array[0].virtual_address, at->size);
         malloc_type_freed(M_NVIDIA, at->size);
     }
 
@@ -1257,8 +1256,7 @@ NvS32 nv_free_contig_pages(
     if (at->attr != VM_MEMATTR_WRITE_BACK)
         os_flush_cpu_cache();
 
-    kmem_free(kernel_map, at->pte_array[0].virtual_address,
-            at->size);
+    kmem_free(kmem_arena, at->pte_array[0].virtual_address, at->size);
     malloc_type_freed(M_NVIDIA, at->size);
 
     free(at->pte_array, M_NVIDIA);
@@ -1321,7 +1319,7 @@ NvS32 nv_alloc_system_pages(
     }
 
     for (i = 0; i < count; i++) {
-        address = kmem_alloc_contig(kernel_map, PAGE_SIZE, flags, 0,
+        address = kmem_alloc_contig(kmem_arena, PAGE_SIZE, flags, 0,
                 sc->dma_mask, PAGE_SIZE, 0, attr);
         if (!address) {
             status = ENOMEM;
@@ -1354,9 +1352,9 @@ NvS32 nv_alloc_system_pages(
         goto failed;
     }
 
-    VM_OBJECT_LOCK(at->object);
+    VM_OBJECT_WLOCK(at->object);
     vm_object_set_memattr(at->object, attr);
-    VM_OBJECT_UNLOCK(at->object);
+    VM_OBJECT_WUNLOCK(at->object);
 
     *private = at;
     SLIST_INSERT_HEAD(&sc->alloc_list, at, list);
@@ -1375,8 +1373,7 @@ failed:
     for (i = 0; i < count; i++) {
         if (at->pte_array[i].virtual_address == 0)
             break;
-        kmem_free(kernel_map,
-                at->pte_array[i].virtual_address, PAGE_SIZE);
+        kmem_free(kmem_arena, at->pte_array[i].virtual_address, PAGE_SIZE);
         malloc_type_freed(M_NVIDIA, PAGE_SIZE);
     }
 
@@ -1405,8 +1402,7 @@ NvS32 nv_free_system_pages(
         os_flush_cpu_cache();
 
     for (i = 0; i < count; i++) {
-        kmem_free(kernel_map,
-                at->pte_array[i].virtual_address, PAGE_SIZE);
+        kmem_free(kmem_arena, at->pte_array[i].virtual_address, PAGE_SIZE);
         malloc_type_freed(M_NVIDIA, PAGE_SIZE);
     }
 
@@ -1683,13 +1679,18 @@ int nvidia_mmap_dev_single(
 {
     struct nvidia_alloc *at;
     nv_stack_t *sp;
+    RM_STATUS rmStatus;
     struct nvidia_softc *sc = nv->os_state;
-    uint32_t i;
+    NvU32 i, prot;
 
     sp = sc->api_sp;
 
     NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, TRUE);
 
+    rmStatus = rm_validate_mmap_request(sp, nv, *offset, size, &prot);
+    if (rmStatus != RM_OK)
+        return EINVAL;
+
     if (IS_UD_OFFSET(nv, *offset, size)) {
         *object = sc->UD_object;
         vm_object_reference(*object);
@@ -1701,9 +1702,6 @@ int nvidia_mmap_dev_single(
         *offset = (*offset - nv->fb->address);
         return 0;
     } else if (IS_REG_OFFSET(nv, *offset, size)) {
-        if (IS_BLACKLISTED_REG_OFFSET(nv, *offset, size)) {
-            return EINVAL;
-        }
         *object = sc->BAR_objects[NV_GPU_BAR_INDEX_REGS];
         vm_object_reference(*object);
         *offset = (*offset - nv->regs->address);
