@@ -55,7 +55,7 @@ void nvidia_sysctl_init(void)
             SYSCTL_CHILDREN(oid_nvidia),
             OID_AUTO,
             "version",
-            CTLFLAG_RD | CTLFLAG_DYN,
+            CTLTYPE_STRING | CTLFLAG_RD | CTLFLAG_DYN,
             (char *)(uintptr_t) pNVRM_ID,
             0,
             "NVIDIA Resource Manager (NVRM) Version");
@@ -326,15 +326,14 @@ int nvidia_sysctl_gpu_vbios(SYSCTL_HANDLER_ARGS)
 int nvidia_sysctl_bus_type(SYSCTL_HANDLER_ARGS)
 {
     struct nvidia_softc *sc = arg1;
-    NvU8 bus_type[4];
+    char *bus_type;
 
     if (nvidia_pci_find_capability(sc->dev, PCIR_CAP_ID_AGP) != 0)
-        sprintf(bus_type, "AGP");
+        bus_type = "AGP";
+    else if (nvidia_pci_find_capability(sc->dev, PCIR_CAP_ID_EXP) != 0)
+        bus_type = "PCIe";
     else
-    if (nvidia_pci_find_capability(sc->dev, PCIR_CAP_ID_EXP) != 0)
-        sprintf(bus_type, "PCI-E");
-    else
-        sprintf(bus_type, "PCI");
+        bus_type = "PCI";
 
     return SYSCTL_OUT(req, bus_type, strlen(bus_type) + 1);
 }
@@ -549,7 +548,7 @@ void nv_sysctl_init(nv_state_t *nv)
             SYSCTL_CHILDREN(oid),
             OID_AUTO,
             "model",
-            CTLFLAG_RD | CTLFLAG_DYN,
+            CTLTYPE_STRING | CTLFLAG_RD | CTLFLAG_DYN,
             (void *) nv, 0,
             nvidia_sysctl_gpu_model,
             "A",
@@ -559,7 +558,7 @@ void nv_sysctl_init(nv_state_t *nv)
             SYSCTL_CHILDREN(oid),
             OID_AUTO,
             "irq",
-            CTLFLAG_RD | CTLFLAG_DYN,
+            CTLTYPE_UINT | CTLFLAG_RD | CTLFLAG_DYN,
             &nv->interrupt_line,
             0,
             "NVIDIA GPU IRQ Number");
@@ -568,7 +567,7 @@ void nv_sysctl_init(nv_state_t *nv)
             SYSCTL_CHILDREN(oid),
             OID_AUTO,
             "vbios",
-            CTLFLAG_RD | CTLFLAG_DYN,
+            CTLTYPE_STRING | CTLFLAG_RD | CTLFLAG_DYN,
             (void *) nv, 0,
             nvidia_sysctl_gpu_vbios,
             "A",
@@ -578,7 +577,7 @@ void nv_sysctl_init(nv_state_t *nv)
             SYSCTL_CHILDREN(oid),
             OID_AUTO,
             "uuid",
-            CTLFLAG_RD | CTLFLAG_DYN,
+            CTLTYPE_STRING | CTLFLAG_RD | CTLFLAG_DYN,
             (void *) nv, 0,
             nvidia_sysctl_gpu_uuid,
             "A",
@@ -588,7 +587,7 @@ void nv_sysctl_init(nv_state_t *nv)
             SYSCTL_CHILDREN(oid),
             OID_AUTO,
             "type",
-            CTLFLAG_RD | CTLFLAG_DYN,
+            CTLTYPE_STRING | CTLFLAG_RD | CTLFLAG_DYN,
             (void *) sc, 0,
             nvidia_sysctl_bus_type,
             "A",
